<?php
require_once "error_object.inc.php";
require_once "parser_global.inc.php";

class AllypageParser extends XMLParserGlobal{

	function __construct($dbcon,$allytable,$playertable,$utablename,$universe) {
		$this->xml_schema = "xml_schema/allypage.xsd";
		//  call super constructor
		$result = parent::__construct($dbcon,$allytable,$playertable,$utablename,$universe);
		if ($result === false) {
			return false;
		}

		return $this;
	}

	public function insert_data($xml_content) {
		$xdoc = $this->validate_header($xml_content,XMLParserGlobal::content_type_allypage);
		if ($xdoc === false) {
			return false;
		}

		// allypage header attribute(s)
		$stats_header = $xdoc->getElementsByTagName("allypage_header"); // there is exactly one
		$allyname     = trim($stats_header->item(0)->getAttribute("name"));
		unset($stats_header);

		// check allyname is not initial
		if (trim($allyname) == "") {
			$this->error_object = new ErrorObject(ErrorObject::severity_error ,"632");
			$this->error_object->add_child_message(new ErrorObject(ErrorObject::severity_error ,"Allyname missing"));
			return false;
		}

		// parse data
		$result = $this->insert_allypage_data($xdoc,$allyname,$this->userid);
		// add error or success messages
		$this->check_result($result);

		return $result;
	}


	private function insert_allypage_data($xdoc,$allyname,$userid) {
		$players = $xdoc->getElementsByTagName("player");
		$players_data = array();

		foreach($players as $player) {
			// extract information from XML file
			$player_data = $this->get_player_data($player);
			// store results
			array_push($players_data, $player_data);
		}
		unset($players);
		unset($xdoc);

		if ($this->update_players_at_database($players_data,$allyname,$userid)) {
			return true;
		} else {
			return false;
		}
	}

	private function get_player_data($player_DOMNode) {
		$return_value = array();
		$return_value["rank"]       = $player_DOMNode->getAttribute("rank");
		$return_value["playername"] = trim($player_DOMNode->getAttribute("playername"));
		$return_value["playerid"]   = $player_DOMNode->getAttribute("playerid");
		$return_value["galaxy"]     = $player_DOMNode->getAttribute("galaxy");
		$return_value["system"]     = $player_DOMNode->getAttribute("system");
		$return_value["planet"]     = $player_DOMNode->getAttribute("planet");
		$return_value["score"]      = $player_DOMNode->getAttribute("score");

		return $return_value;
	}


	private function update_players_at_database(array $players_data, $allyname, $userid) {
		// check if alliance exists
		$query = "SELECT id, allyname FROM $this->allytable WHERE allyname = '".mysql_real_escape_string($allyname)."'";
		$res = $this->run_mysql_query($query);
		if (!$res) {
			$this->error_object = new ErrorObject(ErrorObject::severity_error , "MySQL error occurred while inserting and updating allyname");
			$this->error_object->add_child_message($this->get_last_mysql_error());
			return false;
		}
		
		if (mysql_num_rows($res) > 0) {
			$line        = mysql_fetch_object($res);
			$alliance_id = $line->id;
		} else {
			// insert or alliance first
			$query = "INSERT INTO $this->allytable (user_id, last_update, allyname) ".
			"VALUES ('".mysql_real_escape_string($userid)."',NOW(),'".mysql_real_escape_string($allyname)."') ON DUPLICATE KEY UPDATE last_update=VALUES(last_update), allyname=VALUES(allyname)";
	
			$res = $this->run_mysql_query($query);
			if (!$res) {
				$this->error_object = new ErrorObject(ErrorObject::severity_error , "MySQL error occurred while inserting and updating allyname");
				$this->error_object->add_child_message($this->get_last_mysql_error());
				return false;
			}
	
			$alliance_id = mysql_insert_id($this->mysql_connection);
		}

		// determine all players which are at the database and have an ogame playerid
		$run_query = false;
		$query = "SELECT id,playername,ogame_playerid FROM $this->playertable WHERE ogame_playerid IN (";
		for ($i=0; $i<count($players_data); $i++) {
			if ($players_data[$i]['playerid'] > 0) {
				$query .= "'".mysql_real_escape_string($players_data[$i]['playerid'])."',";
				$run_query = true;
			}
		}
		if ($run_query) {
			// remove the , behind the last query part
			$query = substr($query,0,strlen($query)-1);
			$query .= ")";


			$res = $this->run_mysql_query($query);
			//echo mysql_num_rows($res)." Entries with ogame playerid found\n";
			if (!$res) {
				$this->error_object = new ErrorObject(ErrorObject::severity_error , "MySQL error occurred while collecting players with ogame playerid");
				$this->error_object->add_child_message($this->get_last_mysql_error());
				return false;
			}
			while ($line = mysql_fetch_object($res)) {
				$player_names_2_ids[strtolower($line->playername)]["id"]       = $line->id;
				$player_names_2_ids[strtolower($line->playername)]["ogame_id"] = $line->ogame_playerid;

				// the playername might have changed meanwhile
				$ogame_playerid_2_galaxytool_id[$line->ogame_playerid] = $line->id;
			}
		}

		// determine the player itself as he might have no ogame playerid at stats (but possibly at the galaxytool)
		// TODO: check if $this->own_ogame_playerid could be used instead
		$query = "SELECT id,playername,ogame_playerid FROM $this->playertable WHERE playername IN (";
		$j = 0;
		for ($i=0; $i<count($players_data); $i++) {
			$players_data[$i]['playername'] = trim($players_data[$i]['playername']);
			if ($players_data[$i]['playerid'] == 0) {
				$query .= "'".mysql_real_escape_string($players_data[$i]['playername'])."',";
				$j++;
			}
		}
		if ($j == 1 && $this->own_ogame_playerid > 0) {
			// do nothing, ogame playerid will be used later, just do not run additional queries
		} elseif ($j > 0) {
			// should only happen if ogame has a bug or any extensions removes the HTML code for the playerid
			// or if the own ogame playerid is not yet known


			// remove the , behind the last query part
			$query = substr($query,0,strlen($query)-1);
			$query .= ")";

			$res = $this->run_mysql_query($query);
			//echo mysql_num_rows($res)." Entries without ogame playerid found\n";
			if (!$res) {
				$this->error_object = new ErrorObject(ErrorObject::severity_error , "MySQL error occurred while collecting player without ogame playerid");
				$this->error_object->add_child_message($this->get_last_mysql_error());
				return false;
			}
			while ($line = mysql_fetch_object($res)) {
				$player_names_2_ids[strtolower($line->playername)]["id"] = $line->id;
				$player_names_2_ids[strtolower($line->playername)]["ogame_id"] = $line->ogame_playerid;
			}
		}

		// insert player data
		$query_collection = "INSERT INTO $this->playertable (id,user_id,rank,alliance_id,points,last_stats_update,playername,homegalaxy,homesystem,homeplanet,ogame_playerid) VALUES ";
		$query_collection2 = "INSERT INTO $this->playertable (id,user_id,rank,alliance_id,points,last_stats_update,playername,homegalaxy,homesystem,homeplanet) VALUES ";
		
		$query_1 = false;
		$query_2 = false;
		for ($i=0; $i<count($players_data); $i++) {

			if (empty($players_data[$i]['playerid']) && $this->own_ogame_playerid > 0) {
				$players_data[$i]['playerid'] = $this->own_ogame_playerid;
			}

			// add query data (id,rank,alliance_id,points,last_stats_update,playername,ogame_playerid)
			if (!empty($players_data[$i]['playerid'])) {
				//echo "Ogame player ID: ".$players_data[$i]['playerid']." ";
				// ogame playerid known here
				if (isset($player_names_2_ids[strtolower($players_data[$i]['playername'])]["ogame_id"])) {
					//echo " |".$player_names_2_ids[strtolower($players_data[$i]['playername'])]["ogame_id"]."| ";
					// database contains that player with ogame playerid
					//echo "database contains that player with ogame playerid ".$players_data[$i]['playerid']."\n";
					$query_2 = true;
					$query_collection2 .= " (".$player_names_2_ids[strtolower($players_data[$i]['playername'])]["id"].", $userid, '".mysql_real_escape_string($players_data[$i]['rank'])."','".mysql_real_escape_string($alliance_id)."','".mysql_real_escape_string($players_data[$i]['score'])."',NOW(),'".mysql_real_escape_string($players_data[$i]['playername'])."', '".mysql_real_escape_string($players_data[$i]['galaxy'])."', '".mysql_real_escape_string($players_data[$i]['system'])."', '".mysql_real_escape_string($players_data[$i]['planet'])."'), ";
				} elseif (isset($ogame_playerid_2_galaxytool_id[$players_data[$i]['playerid']])) {
					//echo "playername changed\n";

					$query_2 = true;
					$query_collection2 .= " (".$ogame_playerid_2_galaxytool_id[$players_data[$i]['playerid']].", $userid, '".mysql_real_escape_string($players_data[$i]['rank'])."','".mysql_real_escape_string($alliance_id)."','".mysql_real_escape_string($players_data[$i]['score'])."',NOW(),'".mysql_real_escape_string($players_data[$i]['playername'])."', '".mysql_real_escape_string($players_data[$i]['galaxy'])."', '".mysql_real_escape_string($players_data[$i]['system'])."', '".mysql_real_escape_string($players_data[$i]['planet'])."'), ";

				} elseif (isset($player_names_2_ids[strtolower($players_data[$i]['playername'])]["id"])) {
					// player is included without ogame playerid
					//echo "player ".$player_names_2_ids[strtolower($players_data[$i]['playername'])]["id"]." is included without ogame playerid\n";
					$query_1 = true;
					$query_collection .= " (".$player_names_2_ids[strtolower($players_data[$i]['playername'])]["id"].",$userid,'".mysql_real_escape_string($players_data[$i]['rank'])."','".mysql_real_escape_string($alliance_id)."','".mysql_real_escape_string($players_data[$i]['score'])."',NOW(),'".mysql_real_escape_string($players_data[$i]['playername'])."', '".mysql_real_escape_string($players_data[$i]['galaxy'])."', '".mysql_real_escape_string($players_data[$i]['system'])."', '".mysql_real_escape_string($players_data[$i]['planet'])."','".intval($players_data[$i]['playerid'])."'), ";
				} else {
					// player new to database
					//echo "player new to database\n";
					$query_1 = true;
					$query_collection .= " (NULL,$userid,'".mysql_real_escape_string($players_data[$i]['rank'])."','".mysql_real_escape_string($alliance_id)."','".mysql_real_escape_string($players_data[$i]['score'])."',NOW(),'".mysql_real_escape_string($players_data[$i]['playername'])."', '".mysql_real_escape_string($players_data[$i]['galaxy'])."', '".mysql_real_escape_string($players_data[$i]['system'])."', '".mysql_real_escape_string($players_data[$i]['planet'])."','".intval($players_data[$i]['playerid'])."'), ";
				}
			} else {
				// ogame playerid unknown (player itself)
				if (isset($player_names_2_ids[strtolower($players_data[$i]['playername'])]["id"])) {
					// player already at database
					//echo "player already at database\n";
					$query_2 = true;
					$query_collection2 .= " (".$player_names_2_ids[strtolower($players_data[$i]['playername'])]["id"].",$userid, '".mysql_real_escape_string($players_data[$i]['rank'])."','".mysql_real_escape_string($alliance_id)."','".mysql_real_escape_string($players_data[$i]['score'])."',NOW(),'".mysql_real_escape_string($players_data[$i]['playername'])."', '".mysql_real_escape_string($players_data[$i]['galaxy'])."', '".mysql_real_escape_string($players_data[$i]['system'])."', '".mysql_real_escape_string($players_data[$i]['planet'])."'), ";
				} else {
					// player new to database
					//echo "player new to database\n";
					$query_2 = true;
					$query_collection2 .= " (NULL,$userid,'".mysql_real_escape_string($players_data[$i]['rank'])."','".mysql_real_escape_string($alliance_id)."','".mysql_real_escape_string($players_data[$i]['score'])."',NOW(),'".mysql_real_escape_string($players_data[$i]['playername'])."', '".mysql_real_escape_string($players_data[$i]['galaxy'])."', '".mysql_real_escape_string($players_data[$i]['system'])."', '".mysql_real_escape_string($players_data[$i]['planet'])."'), ";
				}
			}
		}

		// remove last "," at the queries
		$query_collection = substr($query_collection,0,strlen($query_collection)-2);
		$query_collection2 = substr($query_collection2,0,strlen($query_collection2)-2);
		$query_collection .= " ON DUPLICATE KEY UPDATE user_id=VALUES(user_id), rank=VALUES(rank), alliance_id=VALUES(alliance_id), points=VALUES(points), last_stats_update=NOW(), playername=VALUES(playername), homegalaxy=VALUES(homegalaxy), homesystem=VALUES(homesystem), homeplanet=VALUES(homeplanet), ogame_playerid=VALUES(ogame_playerid) ";
		$query_collection2 .= " ON DUPLICATE KEY UPDATE user_id=VALUES(user_id), rank=VALUES(rank), alliance_id=VALUES(alliance_id), points=VALUES(points), last_stats_update=NOW(), playername=VALUES(playername), homegalaxy=VALUES(homegalaxy), homesystem=VALUES(homesystem), homeplanet=VALUES(homeplanet) ";

		if ($query_1 === true) {
			$res = $this->run_mysql_query($query_collection);
			if (!$res) {
				$this->error_object = new ErrorObject(ErrorObject::severity_error , "MySQL error occurred while inserting or updating players with ogame playerid");
				$this->error_object->add_child_message($this->get_last_mysql_error());
				return false;
			}
		}

		if ($query_2 === true) {
			$res = $this->run_mysql_query($query_collection2);
			if (!$res) {
				$this->error_object = new ErrorObject(ErrorObject::severity_error , "MySQL error occurred while inserting or updating players without ogame playerid");
				$this->error_object->add_child_message($this->get_last_mysql_error());
				return false;
			}
		}

		return true;
	}

}





