<?php

class XMLParserGlobal{

	const content_type_allypage     = "allypage";
	const content_type_stats        = "stats";
	const content_type_galaxyview   = "galaxyview";
	const content_type_planetinfo   = "planetinfo";
	const content_type_reports      = "reports";
	const content_type_ally_stats   = "ally_stats";
	const content_type_player_stats = "player_stats";

	protected $xml_schema         = "";
	protected $playertable        = "";
	protected $allytable          = "";
	protected $utablename         = "";
	protected $mysql_error        = null;
	protected $error_object       = null;
	protected $own_ogame_playerid = 0;
	protected $universe           = "";
	protected $debug              = false;
	protected $userid             = 0;
	protected $content_type       = "";


	function __construct($dbcon,$allytable,$playertable,$utablename,$universe) {
		if (!$dbcon) return false;
		if (trim($allytable) == "") return false;
		if (trim($playertable) == "") return false;
		if (trim($universe) == "") return false;

		$this->allytable   = $allytable;
		$this->playertable = $playertable;
		$this->utablename  = $utablename;
		$this->universe    = $universe;

		$this->mysql_connection = $dbcon;

		return $this;
	}

	public function get_error_object() {
		return $this->error_object;
	}



	protected function validate_header($xml_content, $exp_content_type) {
		global $_SERVER;
		
		if (trim($xml_content) == "") {
			return false;
		}
		if ($exp_content_type != XMLParserGlobal::content_type_allypage &&
		$exp_content_type != XMLParserGlobal::content_type_stats &&
		$exp_content_type != XMLParserGlobal::content_type_galaxyview &&
		$exp_content_type != XMLParserGlobal::content_type_planetinfo &&
		$exp_content_type != XMLParserGlobal::content_type_reports ) {
			return false;
		}

		libxml_use_internal_errors(true); // use own error handling
		$this->exp_content_type = $exp_content_type;

		$xdoc = new DomDocument;
		@$xdoc->loadXML($xml_content);
		if ($xdoc === false) {
			// not XML
			return false;
		}
		if (@$xdoc->schemaValidate($this->xml_schema) === false) {
			// XML file is invalid
			$this->error_object = new ErrorObject(ErrorObject::severity_info ,"703");

			$errors = libxml_get_errors();
			foreach ($errors as $error) {
				$this->error_object->add_child_message(new ErrorObject(ErrorObject::severity_error, $this->display_xml_error($error, $xml_content)));
			}
			libxml_clear_errors();

			return false;
		}

		// get header attributes
		$header_items = $xdoc->getElementsByTagName("header"); // there is exactly one
		$language     = $header_items->item(0)->getAttribute("language");
		// $version      = $header_items->item(0)->getAttribute("version"); // currently no need for this
		$universe     = $header_items->item(0)->getAttribute("universe");
		$debug        = $header_items->item(0)->getAttribute("debug");
		$username     = $header_items->item(0)->getAttribute("username");
		$password     = $header_items->item(0)->getAttribute("password");
		$content_type = $header_items->item(0)->getAttribute("content_type");
		unset($header_items);

		// load language file
		switch ($language) {
			case "german"     : require_once "../languages/german.inc.php"; break;
			case "english"    : require_once "../languages/english.inc.php"; break;
			case "spanish"    : require_once "../languages/spanish.inc.php"; break;
			case "dutch"      : require_once "../languages/dutch.inc.php"; break;
			case "balkan"     : require_once "../languages/balkan.inc.php"; break;
			case "french"     : require_once "../languages/french.inc.php"; break;
			case "portugues"  : require_once "../languages/portugues.inc.php"; break;
			case "polish"     : require_once "../languages/polish.inc.php"; break;
			case "italian"    : require_once "../languages/italian.inc.php"; break;
			case "turkish"    : require_once "../languages/turkish.inc.php"; break;
			case "danish"     : require_once "../languages/danish.inc.php"; break;
			case "brazilian"  : require_once "../languages/brazilian.inc.php"; break;
			case "russian"    : require_once "../languages/russian.inc.php"; break;
			case "swedish"    : require_once "../languages/swedish.inc.php"; break;
			case "greek"      : require_once "../languages/greek.inc.php"; break;
			case "romanian"   : require_once "../languages/romanian.inc.php"; break;
			case "hungarian"  : require_once "../languages/hungarian.inc.php"; break;
			case "czech"      : require_once "../languages/czech.inc.php"; break;
			case "korean"     : require_once "../languages/korean.inc.php"; break;
			case "norwegian"  : require_once "../languages/norwegian.inc.php"; break;
			case "taiwan"     : require_once "../languages/taiwan.inc.php"; break;
			case "japan"      : require_once "../languages/japanese.inc.php"; break;
			case "chinese"    : require_once "../languages/chinese.inc.php"; break;
			case "bulgarian"  : require_once "../languages/bulgarian.inc.php"; break;
			case "lithuanian" : require_once "../languages/lithuanian.inc.php"; break;
			case "latvian"    : require_once "../languages/latvian.inc.php"; break;
			case "finnish"    : require_once "../languages/finnish.inc.php"; break;
			case "slovak"     : require_once "../languages/slovak.inc.php"; break;
			case "croatian"   : require_once "../languages/croatian.inc.php"; break;
			case "serbian"    : require_once "../languages/serbian.inc.php"; break;
			default : require_once "../languages/english.inc.php"; break;
		}
	
		// check universe
		if ($universe != $this->universe && (!isset($_SERVER['HTTP_HOST']) || strpos($_SERVER['HTTP_HOST'].$_SERVER['PHP_SELF'],"galaxytool-hosting.eu/galatool_beta") === false)) {
			$this->error_object = new ErrorObject(ErrorObject::severity_info ,"701");
			$message = str_replace("&1",$universe,PARSER_WRONG_UNIVERSE);
			$message = str_replace("&2",$this->universe,$message);
			$this->error_object->add_child_message(new ErrorObject(ErrorObject::severity_error ,$message));
			return false;
		}

		// check content type
		if ($content_type != $exp_content_type) {
			if (($content_type == XMLParserGlobal::content_type_ally_stats || $content_type == XMLParserGlobal::content_type_player_stats) && $exp_content_type == XMLParserGlobal::content_type_stats) {
				$this->content_type = $content_type;
			} elseif ($content_type == XMLParserGlobal::content_type_planetinfo && $exp_content_type == XMLParserGlobal::content_type_reports ) { 
				$this->content_type = $content_type;
			} else {
				$this->error_object = new ErrorObject(ErrorObject::severity_info ,$this->get_error_code($exp_content_type));
				return false;
			}
		} else {
			$this->content_type = $content_type;
		}

		// set debug options
		if ($debug == "true") {
			$this->debug = true;
		} else {
			$this->debug = false;
		}

		// check permissions
		$query = "SELECT id, caninsert, ogame_playerid FROM $this->utablename WHERE username='".mysql_real_escape_string(urldecode($username),$this->mysql_connection)."' AND userpass='".mysql_real_escape_string($password)."' AND status='active'";
		$res = $this->run_mysql_query($query);
		if ($res !== false && mysql_num_rows($res) > 0) {

			$line                     = mysql_fetch_object($res);
			$this->userid             = $line->id;
			$allow_insert             = $line->caninsert;
			$this->own_ogame_playerid = $line->ogame_playerid;

			if ($allow_insert != "true") {
				// mssing permissions
				$this->error_object = new ErrorObject(ErrorObject::severity_info ,$this->get_error_code($exp_content_type));
				$this->error_object->add_child_message(new ErrorObject(ErrorObject::severity_error ,"No permission"));
				return false;
			}
		} else {
			// wrong username / password
			$this->error_object = new ErrorObject(ErrorObject::severity_info ,$this->get_error_code($exp_content_type));
			$this->error_object->add_child_message(new ErrorObject(ErrorObject::severity_error ,L_LOGINFAILURE));
			return false;
		}

		return $xdoc;
	}

	protected function check_result($result) {
		if ($result === true) {
			// no errors occured

			
			if ($this->error_object == null) {
				$this->error_object = new ErrorObject(ErrorObject::severity_info ,$this->get_success_code($this->exp_content_type));
				return;
			}

			// but messages occured
			$old_error_object = $this->error_object;
			if ($old_error_object->get_worst_severity() == ErrorObject::severity_info ) {
				// Info
				$this->error_object = new ErrorObject(ErrorObject::severity_info ,$this->get_success_code($this->exp_content_type));
				$this->error_object->add_child_message($old_error_object);
			} else {
				// Warning and Errors
				$this->error_object = new ErrorObject(ErrorObject::severity_info ,$this->get_error_code($this->exp_content_type));
				$this->error_object->add_child_message($old_error_object);
			}

		} else {
			// messages occured
			if ($this->error_object == null) {
				$old_error_object = new ErrorObject(ErrorObject::severity_error , "Unexpected error occurred");
			} else {
				$old_error_object = $this->error_object;
			}
			$this->error_object = new ErrorObject(ErrorObject::severity_info ,$this->get_error_code($this->exp_content_type));
			$this->error_object->add_child_message($old_error_object);
		}

	}

	/*
	* Status Codes:
	- 601: galaxy view inserted
	- 602: problem with inserting galaxy view
	- 611: report inserted
	- 612: at least one report was wrong
	- 621: stats updated
	- 622: stats not updated
	- 631: allyhistory updated
	- 632: allyhistory not updated
	- 700: no content
	- 701: wrong universe
	- 702: toolbar outdated
	- 703: invalid XML
	*/
	protected function get_error_code($exp_content_type) {
		switch ($exp_content_type) {
			case XMLParserGlobal::content_type_allypage:    return "632";
			case XMLParserGlobal::content_type_stats:       return "622";
			case XMLParserGlobal::content_type_galaxyview:  return "602";
			case XMLParserGlobal::content_type_planetinfo:  return "612";
			case XMLParserGlobal::content_type_reports:     return "612";

			default: return "000";
		}
	}

	protected function get_success_code($exp_content_type) {
		switch ($exp_content_type) {
			case XMLParserGlobal::content_type_allypage:    return "631";
			case XMLParserGlobal::content_type_stats:       return "621";
			case XMLParserGlobal::content_type_galaxyview:  return "601";
			case XMLParserGlobal::content_type_planetinfo:  return "611";
			case XMLParserGlobal::content_type_reports:     return "611";

			default: return "000";
		}
	}


	protected function run_mysql_query($query) {
		$this->set_last_mysql_error(null); // clear mysql errors

		$res = mysql_query($query,$this->mysql_connection);
		if (!$res) {
			//if ($this->debug === true) {
				$error_object = new ErrorObject(ErrorObject::severity_error, mysql_error());
				$error_object->add_child_message(new ErrorObject(ErrorObject::severity_info , $query));
				$this->set_last_mysql_error($error_object);
			//}

			return false;
		} else {
				return $res;
		}
	}

	protected function set_last_mysql_error(ErrorObject $errorObject = NULL) {
		$this->mysql_error = $errorObject;
	}

	protected function get_last_mysql_error() {
		return $this->mysql_error;
	}

	private function display_xml_error($error, $xml) {
		//$return  = $xml[$error->line - 1] . "\n";
		//$return .= str_repeat('-', $error->column) . "^\n";
		$return = "";

		switch ($error->level) {
			case LIBXML_ERR_WARNING:
				$return .= "Warning $error->code: ";
				break;
			case LIBXML_ERR_ERROR:
				$return .= "Error $error->code: ";
				break;
			case LIBXML_ERR_FATAL:
				$return .= "Fatal Error $error->code: ";
				break;
		}

		$return .= trim($error->message) .
		"\n  Line: $error->line" .
		"\n  Column: $error->column";

		if ($error->file) {
			$return .= "\n  File: $error->file \n";
		}

		return $return;
	}

}

