<?php

/**
 * As soon as the current URL does not fit the URL at $_SESSION['s_serverurl'] this function is called
 * - check if there is a session for the new URL -> no = timeout
 * - make a backup of the previous session information
 * - copy information from new URL session to current session files
 * 
 * URL is not set directly at the array - md5() of the URL is used
 *
 * @param String $new_url New URL which was opened
 * @return false if the new URL is not at our session buffer; else true
 */
function swap_Session($new_url) {
	$new_url = str_replace("secret/","",$new_url);

	$url_hash_new = md5($new_url);

	if (!isset($_SESSION[$url_hash_new])) {
		// we do not know the session where we want to restore from
		return false;
	}

	if (isset($_SESSION['s_serverurl'])) {
		// the other session may have been destroyed due to logout
		backup_session();
	}

	// report.php - unset all search parameter
	unset($_SESSION['s_spioarray']);
	unset($_SESSION['s_spiogala']);
	unset($_SESSION['s_spiosystem1']);
	unset($_SESSION['s_spiosystem2']);
	unset($_SESSION['s_hits']);
	unset($_SESSION['s_datevalue']);
	unset($_SESSION['s_time1']);
	unset($_SESSION['s_time2']);
	unset($_SESSION['s_searchtype']);

	// show.php
	unset_page_parameter("show.php");


	// Now copy the new values into the session
	$_SESSION['s_systems']          = $_SESSION[$url_hash_new]['s_systems'];
	$_SESSION['s_lastweek']         = $_SESSION[$url_hash_new]['s_lastweek'];
	$_SESSION['s_too_old']          = $_SESSION[$url_hash_new]['s_too_old'];
	$_SESSION['s_req_user']         = $_SESSION[$url_hash_new]['s_req_user'];
	$_SESSION['s_notices']          = $_SESSION[$url_hash_new]['s_notices'];
	$_SESSION['s_players']          = $_SESSION[$url_hash_new]['s_players'];
	$_SESSION['s_allies']           = $_SESSION[$url_hash_new]['s_allies'];
	$_SESSION['s_reports']          = $_SESSION[$url_hash_new]['s_reports'];
	$_SESSION['s_umods']            = $_SESSION[$url_hash_new]['s_umods'];
	$_SESSION['s_moons']            = $_SESSION[$url_hash_new]['s_moons'];
	$_SESSION['s_raidable']         = $_SESSION[$url_hash_new]['s_raidable'];
	$_SESSION['s_empty_systems']    = $_SESSION[$url_hash_new]['s_empty_systems'];
	$_SESSION['s_full_systems']     = $_SESSION[$url_hash_new]['s_full_systems'];
	$_SESSION['s_minimumresources'] = $_SESSION[$url_hash_new]['s_minimumresources'];
	$_SESSION['s_spioarray']        = $_SESSION[$url_hash_new]['s_spioarray'];
	$_SESSION['s_saves']            = $_SESSION[$url_hash_new]['s_saves'];

	// personal information
	$_SESSION['s_username']        = $_SESSION[$url_hash_new]['s_username'];
	$_SESSION['s_userid']          = $_SESSION[$url_hash_new]['s_userid'];
	$_SESSION['lang']              = $_SESSION[$url_hash_new]['lang']; // no s_ here
	$_SESSION['s_ogame_playerid']  = $_SESSION[$url_hash_new]['s_ogame_playerid'];
	$_SESSION['s_loggedin']        = $_SESSION[$url_hash_new]['s_loggedin'];
	$_SESSION['s_stylepath']       = $_SESSION[$url_hash_new]['s_stylepath'];
	$_SESSION['s_timezone_offset'] = $_SESSION[$url_hash_new]['s_timezone_offset'];

	// restrictions
	$_SESSION['s_isadmin']     = $_SESSION[$url_hash_new]['s_isadmin'];
	$_SESSION['s_serverurl']   = $_SESSION[$url_hash_new]['s_serverurl'];
	$_SESSION['s_cansearch']   = $_SESSION[$url_hash_new]['s_cansearch'];
	$_SESSION['s_caninsert']   = $_SESSION[$url_hash_new]['s_caninsert'];
	$_SESSION['s_statusview']  = $_SESSION[$url_hash_new]['s_statusview'];
	$_SESSION['s_probeview']   = $_SESSION[$url_hash_new]['s_probeview'];
	$_SESSION['s_userman']     = $_SESSION[$url_hash_new]['s_userman'];
	$_SESSION['s_allyhistory'] = $_SESSION[$url_hash_new]['s_allyhistory'];
	$_SESSION['s_diplomatic']  = $_SESSION[$url_hash_new]['s_diplomatic'];

	// personal options
	$_SESSION['s_hits']             = $_SESSION[$url_hash_new]['s_hits'];
	$_SESSION['s_statspage']        = $_SESSION[$url_hash_new]['s_statspage'];
	$_SESSION['s_additional_links'] = $_SESSION[$url_hash_new]['s_additional_links'];
	$_SESSION['s_global_links']     = $_SESSION[$url_hash_new]['s_global_links'];

	// BB Code settings
	$_SESSION['s_bb_coordinates']  = $_SESSION[$url_hash_new]['s_bb_coordinates'];
	$_SESSION['s_bb_moon']         = $_SESSION[$url_hash_new]['s_bb_moon'];
	$_SESSION['s_bb_debris']       = $_SESSION[$url_hash_new]['s_bb_debris'];
	$_SESSION['s_bb_ally']         = $_SESSION[$url_hash_new]['s_bb_ally'];
	$_SESSION['s_bb_allydetails']  = $_SESSION[$url_hash_new]['s_bb_allydetails'];
	$_SESSION['s_bb_playername']   = $_SESSION[$url_hash_new]['s_bb_playername'];
	$_SESSION['s_bb_banned']       = $_SESSION[$url_hash_new]['s_bb_banned'];
	$_SESSION['s_bb_vacation']     = $_SESSION[$url_hash_new]['s_bb_vacation'];
	$_SESSION['s_bb_noob']         = $_SESSION[$url_hash_new]['s_bb_noob'];
	$_SESSION['s_bb_inactive']     = $_SESSION[$url_hash_new]['s_bb_inactive'];
	$_SESSION['s_bb_longinactive'] = $_SESSION[$url_hash_new]['s_bb_longinactive'];

	// view.php
	$_SESSION['s_gv_system'] = $_SESSION[$url_hash_new]['s_gv_system'];
	$_SESSION['s_gv_galaxy'] = $_SESSION[$url_hash_new]['s_gv_galaxy'];


	// websim data
	$_SESSION['s_waffentech'] = $_SESSION[$url_hash_new]['s_waffentech'];
	$_SESSION['s_schildtech'] = $_SESSION[$url_hash_new]['s_schildtech'];
	$_SESSION['s_rpz']        = $_SESSION[$url_hash_new]['s_rpz'];
	$_SESSION['s_vbt']        = $_SESSION[$url_hash_new]['s_vbt'];
	$_SESSION['s_hra']        = $_SESSION[$url_hash_new]['s_hra'];
	$_SESSION['s_impulse']    = $_SESSION[$url_hash_new]['s_impulse'];

	return true;
}

/**
 * Put all session information into the session handler array
 *
 */
function backup_session() {
	$url_hash = md5($_SESSION['s_serverurl']);
	// backup current values (overwrite or create)
	// index page with all gathered information
	$_SESSION[$url_hash]['s_systems']          = $_SESSION['s_systems'];
	$_SESSION[$url_hash]['s_lastweek']         = $_SESSION['s_lastweek'];
	$_SESSION[$url_hash]['s_too_old']          = $_SESSION['s_too_old'];
	$_SESSION[$url_hash]['s_req_user']         = $_SESSION['s_req_user'];
	$_SESSION[$url_hash]['s_notices']          = $_SESSION['s_notices'];
	$_SESSION[$url_hash]['s_players']          = $_SESSION['s_players'];
	$_SESSION[$url_hash]['s_allies']           = $_SESSION['s_allies'];
	$_SESSION[$url_hash]['s_reports']          = $_SESSION['s_reports'];
	$_SESSION[$url_hash]['s_umods']            = $_SESSION['s_umods'];
	$_SESSION[$url_hash]['s_moons']            = $_SESSION['s_moons'];
	$_SESSION[$url_hash]['s_raidable']         = $_SESSION['s_raidable'];
	$_SESSION[$url_hash]['s_empty_systems']    = $_SESSION['s_empty_systems'];
	$_SESSION[$url_hash]['s_full_systems']     = $_SESSION['s_full_systems'];
	$_SESSION[$url_hash]['s_minimumresources'] = $_SESSION['s_minimumresources'];
	$_SESSION[$url_hash]['s_spioarray']        = $_SESSION['s_spioarray'];
	$_SESSION[$url_hash]['s_saves']            = $_SESSION['s_saves'];

	// personal information
	$_SESSION[$url_hash]['s_username']        = $_SESSION['s_username'];
	$_SESSION[$url_hash]['s_userid']          = $_SESSION['s_userid'];
	$_SESSION[$url_hash]['lang']              = $_SESSION['lang']; // no s_ here
	$_SESSION[$url_hash]['s_ogame_playerid']  = $_SESSION['s_ogame_playerid'];
	$_SESSION[$url_hash]['s_loggedin']        = $_SESSION['s_loggedin'];
	$_SESSION[$url_hash]['s_stylepath']       = $_SESSION['s_stylepath'];
	$_SESSION[$url_hash]['s_timezone_offset'] = $_SESSION['s_timezone_offset'];

	// restrictions
	$_SESSION[$url_hash]['s_isadmin']     = $_SESSION['s_isadmin'];
	$_SESSION[$url_hash]['s_serverurl']   = $_SESSION['s_serverurl'];
	$_SESSION[$url_hash]['s_cansearch']   = $_SESSION['s_cansearch'];
	$_SESSION[$url_hash]['s_caninsert']   = $_SESSION['s_caninsert'];
	$_SESSION[$url_hash]['s_statusview']  = $_SESSION['s_statusview'];
	$_SESSION[$url_hash]['s_probeview']   = $_SESSION['s_probeview'];
	$_SESSION[$url_hash]['s_userman']     = $_SESSION['s_userman'];
	$_SESSION[$url_hash]['s_allyhistory'] = $_SESSION['s_allyhistory'];
	$_SESSION[$url_hash]['s_diplomatic']  = $_SESSION['s_diplomatic'];

	// personal options
	$_SESSION[$url_hash]['s_hits']             = $_SESSION['s_hits'];
	$_SESSION[$url_hash]['s_statspage']        = $_SESSION['s_statspage'];
	$_SESSION[$url_hash]['s_additional_links'] = $_SESSION['s_additional_links'];
	$_SESSION[$url_hash]['s_global_links']     = $_SESSION['s_global_links'];

	// BB Code settings
	$_SESSION[$url_hash]['s_bb_coordinates']  = $_SESSION['s_bb_coordinates'];
	$_SESSION[$url_hash]['s_bb_moon']         = $_SESSION['s_bb_moon'];
	$_SESSION[$url_hash]['s_bb_debris']       = $_SESSION['s_bb_debris'];
	$_SESSION[$url_hash]['s_bb_ally']         = $_SESSION['s_bb_ally'];
	$_SESSION[$url_hash]['s_bb_allydetails']  = $_SESSION['s_bb_allydetails'];
	$_SESSION[$url_hash]['s_bb_playername']   = $_SESSION['s_bb_playername'];
	$_SESSION[$url_hash]['s_bb_banned']       = $_SESSION['s_bb_banned'];
	$_SESSION[$url_hash]['s_bb_vacation']     = $_SESSION['s_bb_vacation'];
	$_SESSION[$url_hash]['s_bb_noob']         = $_SESSION['s_bb_noob'];
	$_SESSION[$url_hash]['s_bb_inactive']     = $_SESSION['s_bb_inactive'];
	$_SESSION[$url_hash]['s_bb_longinactive'] = $_SESSION['s_bb_longinactive'];

	// view.php
	$_SESSION[$url_hash]['s_gv_galaxy'] = $_SESSION['s_gv_galaxy'];
	$_SESSION[$url_hash]['s_gv_system'] = $_SESSION['s_gv_system'];

	// websim data
	$_SESSION[$url_hash]['s_waffentech'] = $_SESSION['s_waffentech'];
	$_SESSION[$url_hash]['s_schildtech'] = $_SESSION['s_schildtech'];
	$_SESSION[$url_hash]['s_rpz']        = $_SESSION['s_rpz'];
	$_SESSION[$url_hash]['s_vbt']        = $_SESSION['s_vbt'];
	$_SESSION[$url_hash]['s_hra']        = $_SESSION['s_hra'];
	$_SESSION[$url_hash]['s_impulse']    = $_SESSION['s_impulse'];


}

/**
 * Unset session values that are used at the given page
 *
 * @param string $page Filename
 */
function unset_page_parameter($page) {
	switch ($page) {
		case "show.php":
			// show.php - unset all search parameter
			unset($_SESSION['s_allysuche']);
			unset($_SESSION['s_spielersuche']);
			unset($_SESSION['s_moonsearch']);
			unset($_SESSION['s_colonysearch']);
			unset($_SESSION['s_sorder1']);
			unset($_SESSION['s_sorder2']);
			unset($_SESSION['s_checked1']);
			unset($_SESSION['s_checked2']);
			unset($_SESSION['s_noticesearch']);
			unset($_SESSION['s_reportsearch']);
			unset($_SESSION['s_saves']);
			unset($_SESSION['s_nchecked1']);
			unset($_SESSION['s_nchecked2']);
			unset($_SESSION['s_rchecked1']);
			unset($_SESSION['s_rchecked2']);
			unset($_SESSION['s_cchecked1']);
			unset($_SESSION['s_cchecked2']);
			unset($_SESSION['s_schecked1']);
			unset($_SESSION['s_schecked2']);
			unset($_SESSION['s_schecked3']);
			unset($_SESSION['s_schecked4']);
			unset($_SESSION['s_galaxie']);
			unset($_SESSION['s_system1']);
			unset($_SESSION['s_system2']);
			unset($_SESSION['s_planet1']);
			unset($_SESSION['s_planet2']);
			unset($_SESSION['s_status1']);
			unset($_SESSION['s_status2']);
			unset($_SESSION['s_status3']);
			unset($_SESSION['s_allystatus2']);
			unset($_SESSION['s_allystatus3']);
			unset($_SESSION['s_allystatus4']);
			unset($_SESSION['s_allystatus5']);
			unset($_SESSION['s_allystatus6']);
			unset($_SESSION['s_allystatus7']);
			unset($_SESSION['s_allystatus8']);
			unset($_SESSION['s_sort']);
			unset($_SESSION['s_datevalue']);
			unset($_SESSION['s_tyear']);
			unset($_SESSION['s_tmonth']);
			unset($_SESSION['s_tday']);
			unset($_SESSION['s_thour']);
			unset($_SESSION['s_minute']);
			unset($_SESSION['s_second']);
			unset($_SESSION['s_metal']);
			unset($_SESSION['s_crystal']);
			unset($_SESSION['s_both']);
			unset($_SESSION['s_minrank']);
			unset($_SESSION['s_minfrank']);
			unset($_SESSION['s_maxrank']);
			unset($_SESSION['s_maxfrank']);
			unset($_SESSION['s_bbcodes']);

			break;

		default: exit("unknown page!");
	}

}

/**
 * Logout of current session - so unset session handler array of that URL
 *
 */
function log_out() {
	if (isset($_SESSION['s_serverurl'])) {
		$url_hash = md5($_SESSION['s_serverurl']);
		unset($_SESSION[$url_hash]);
	}
}

?>